@@ -37,6 +37,7 @@
 #include <ofono/voicecall.h>
 
 #include "common.h"
+#include "hfp.h"
 
 #include "hfpmodem.h"
 #include "slc.h"
@@ -45,6 +46,7 @@
 #define POLL_CLCC_DELAY 50
 #define EXPECT_RELEASE_DELAY 50
 #define CLIP_TIMEOUT 500
+#define EXPECT_RING_DELAY 200
 
 static const char *none_prefix[] = { NULL };
 static const char *clcc_prefix[] = { "+CLCC:", NULL };
@@ -447,7 +449,7 @@ static void hfp_hold_all_active(struct ofono_voicecall *vc,
 {
 	struct voicecall_data *vd = ofono_voicecall_get_data(vc);
 
-	if (vd->ag_mpty_features & AG_CHLD_2) {
+	if (vd->ag_mpty_features & HFP_AG_CHLD_2) {
 		hfp_template("AT+CHLD=2", vc, generic_cb, 0, cb, data);
 		return;
 	}
@@ -461,7 +463,7 @@ static void hfp_release_all_held(struct ofono_voicecall *vc,
 	struct voicecall_data *vd = ofono_voicecall_get_data(vc);
 	unsigned int held_status = 1 << CALL_STATUS_HELD;
 
-	if (vd->ag_mpty_features & AG_CHLD_0) {
+	if (vd->ag_mpty_features & HFP_AG_CHLD_0) {
 		hfp_template("AT+CHLD=0", vc, generic_cb, held_status,
 				cb, data);
 		return;
@@ -476,7 +478,7 @@ static void hfp_set_udub(struct ofono_voicecall *vc,
 	struct voicecall_data *vd = ofono_voicecall_get_data(vc);
 	unsigned int incoming_or_waiting = 1 << CALL_STATUS_WAITING;
 
-	if (vd->ag_mpty_features & AG_CHLD_0) {
+	if (vd->ag_mpty_features & HFP_AG_CHLD_0) {
 		hfp_template("AT+CHLD=0", vc, generic_cb, incoming_or_waiting,
 				cb, data);
 		return;
@@ -498,6 +500,19 @@ static gboolean expect_release(gpointer user_data)
 	return FALSE;
 }
 
+static gboolean expect_ring(gpointer user_data)
+{
+	struct ofono_voicecall *vc = user_data;
+	struct voicecall_data *vd = ofono_voicecall_get_data(vc);
+
+	g_at_chat_send(vd->chat, "AT+CLCC", clcc_prefix,
+				clcc_poll_cb, vc, NULL);
+
+	vd->clip_source = 0;
+
+	return FALSE;
+}
+
 static void release_all_active_cb(gboolean ok, GAtResult *result,
 							gpointer user_data)
 {
@@ -528,7 +543,7 @@ static void hfp_release_all_active(struct ofono_voicecall *vc,
 {
 	struct voicecall_data *vd = ofono_voicecall_get_data(vc);
 
-	if (vd->ag_mpty_features & AG_CHLD_1) {
+	if (vd->ag_mpty_features & HFP_AG_CHLD_1) {
 		hfp_template("AT+CHLD=1", vc, release_all_active_cb, 0x1, cb,
 									data);
 		return;
@@ -559,7 +574,7 @@ static void hfp_release_specific(struct ofono_voicecall *vc, int id,
 	struct release_id_req *req = NULL;
 	char buf[32];
 
-	if (!(vd->ag_mpty_features & AG_CHLD_1x))
+	if (!(vd->ag_mpty_features & HFP_AG_CHLD_1x))
 		goto error;
 
 	req = g_try_new0(struct release_id_req, 1);
@@ -590,7 +605,7 @@ static void hfp_private_chat(struct ofono_voicecall *vc, int id,
 	struct voicecall_data *vd = ofono_voicecall_get_data(vc);
 	char buf[32];
 
-	if (vd->ag_mpty_features & AG_CHLD_2x) {
+	if (vd->ag_mpty_features & HFP_AG_CHLD_2x) {
 		snprintf(buf, sizeof(buf), "AT+CHLD=2%d", id);
 
 		hfp_template(buf, vc, generic_cb, 0, cb, data);
@@ -606,7 +621,7 @@ static void hfp_create_multiparty(struct ofono_voicecall *vc,
 {
 	struct voicecall_data *vd = ofono_voicecall_get_data(vc);
 
-	if (vd->ag_mpty_features & AG_CHLD_3) {
+	if (vd->ag_mpty_features & HFP_AG_CHLD_3) {
 		hfp_template("AT+CHLD=3", vc, generic_cb, 0, cb, data);
 
 		return;
@@ -625,7 +640,7 @@ static void hfp_transfer(struct ofono_voicecall *vc,
 	 */
 	unsigned int transfer = 0x1 | 0x2 | 0x4 | 0x8;
 
-	if (vd->ag_mpty_features & AG_CHLD_4) {
+	if (vd->ag_mpty_features & HFP_AG_CHLD_4) {
 		hfp_template("AT+CHLD=4", vc, generic_cb, transfer, cb, data);
 
 		return;
@@ -639,8 +654,10 @@ static void hfp_send_dtmf(struct ofono_voicecall *vc, const char *dtmf,
 {
 	struct voicecall_data *vd = ofono_voicecall_get_data(vc);
 	struct change_state_req *req = g_try_new0(struct change_state_req, 1);
+	int len = strlen(dtmf);
 	char *buf;
 	int s;
+	int i;
 
 	if (req == NULL)
 		goto error;
@@ -650,12 +667,15 @@ static void hfp_send_dtmf(struct ofono_voicecall *vc, const char *dtmf,
 	req->data = data;
 	req->affected_types = 0;
 
-	/* strlen("AT+VTS=) = 7 + NULL */
-	buf = g_try_new(char, strlen(dtmf) + 8);
+	/* strlen("AT") + (n-1) * strlen("+VTS=T;") + strlen(+VTS=T) + null */
+	buf = g_try_new(char, len * 7 + 2);
 	if (buf == NULL)
 		goto error;
 
-	sprintf(buf, "AT+VTS=%s", dtmf);
+	s = sprintf(buf, "AT+VTS=%c", dtmf[0]);
+
+	for (i = 1; i < len; i++)
+		s += sprintf(buf + s, ";+VTS=%c", dtmf[i]);
 
 	s = g_at_chat_send(vd->chat, buf, none_prefix,
 				generic_cb, req, g_free);
@@ -751,6 +771,11 @@ static void ring_notify(GAtResult *result, gpointer user_data)
 	struct ofono_call *call;
 	GSList *waiting;
 
+	if (vd->clip_source) {
+		g_source_remove(vd->clip_source);
+		vd->clip_source = 0;
+	}
+
 	/* RING can repeat, ignore if we already have an incoming call */
 	if (g_slist_find_custom(vd->calls,
 				GINT_TO_POINTER(CALL_STATUS_INCOMING),
@@ -975,7 +1000,15 @@ static void ciev_callsetup_notify(struct ofono_voicecall *vc,
 		break;
 
 	case 1:
-		/* Handled in RING/CCWA */
+		/*
+		 * Handled in RING/CCWA most of the time, however sometimes
+		 * the call is answered before the RING unsolicited
+		 * notification has a chance to be generated on the device.
+		 * In this case, we use a failsafe CLCC poll in expect_ring
+		 * callback.
+		 * */
+		vd->clip_source = g_timeout_add(EXPECT_RING_DELAY,
+							expect_ring, vc);
 		break;
 
 	case 2:
