@@ -48,6 +48,7 @@
 #include <ofono/call-volume.h>
 #include <ofono/handsfree.h>
 #include <ofono/handsfree-audio.h>
+#include <ofono/siri.h>
 
 #include <drivers/atmodem/atutil.h>
 #include <drivers/hfpmodem/slc.h>
@@ -70,6 +71,7 @@ struct hfp {
 	struct ofono_handsfree_card *card;
 };
 
+static const char *none_prefix[] = { NULL };
 static GDBusClient *bluez = NULL;
 
 static void hfp_debug(const char *str, void *user_data)
@@ -296,6 +298,7 @@ static void hfp_pre_sim(struct ofono_modem *modem)
 	ofono_netreg_create(modem, 0, "hfpmodem", &hfp->info);
 	ofono_handsfree_create(modem, 0, "hfpmodem", &hfp->info);
 	ofono_call_volume_create(modem, 0, "hfpmodem", &hfp->info);
+	ofono_siri_create(modem, 0, "hfpmodem", &hfp->info);
 }
 
 static void hfp_post_sim(struct ofono_modem *modem)
@@ -345,7 +348,7 @@ static void bcs_notify(GAtResult *result, gpointer user_data)
 	sprintf(str, "AT+BCS=%d", value);
 
 done:
-	g_at_chat_send(info->chat, str, NULL, NULL, NULL, NULL);
+	g_at_chat_send(info->chat, str, none_prefix, NULL, NULL, NULL);
 }
 
 static int hfp16_card_probe(struct ofono_handsfree_card *card,
